﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS.DefaultRule {
  [Item("MajorDefaultRule", "Description missing")]
  [StorableClass]
  public class MajorDefaultRule : DefaultRuleOperator, IStochasticOperator {

    #region Parameter Properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    #endregion

    [StorableConstructor]
    protected MajorDefaultRule(bool deserializing) : base(deserializing) { }
    protected MajorDefaultRule(MajorDefaultRule original, Cloner cloner)
      : base(original, cloner) {
    }
    public MajorDefaultRule()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MajorDefaultRule(this, cloner);
    }

    public override IOperation Apply() {
      if (NichingParameter.ActualValue == null) {
        NichingParameter.ActualValue = new BoolValue(false);
        NichesParameter.ActualValue = new IntValue(1);
        var problemData = GAssistNichesProblemDataParameter.ActualValue;
        var trainingIndices = problemData.TrainingIndices;
        var actions = problemData.FetchAction(trainingIndices);

        Dictionary<IGAssistNiche, int> classCount = new Dictionary<IGAssistNiche, int>(actions.First().Comparer);
        foreach (var niche in GAssistNichesProblemDataParameter.ActualValue.GetPossibleNiches()) {
          classCount[niche] = 0;
        }
        foreach (var action in actions) {
          classCount[action] += 1;
        }
        var maxAction = classCount.MaxItems(x => x.Value).ToList();
        var index = RandomParameter.ActualValue.Next(0, maxAction.Count);
        DefaultClassParameter.ActualValue = maxAction[index].Key;
      }
      return base.Apply();
    }
  }
}
